#  R-Version:    R 2.12.1  (2010-12-16)

#  Required input files:
#
# Two files contain the dataset used in the analysis and can
#  be obtained from GEO:
#
#    GSE31519_complete_dataset.txt 
#    GSE31519_TNBC_SampleInfo_BCR.txt 
#   
# How to obtain:
#
# GSE31519_TNBC_SampleInfo_BCR.txt  is available from
# ftp://ftp.ncbi.nih.gov/pub/geo/DATA/supplementary/series/GSE31519/GSE31519%5FTNBC%5FSampleInfo%5FBCR%2Etxt%2Egz
#
# GSE31519_complete_dataset.txt  is available from
# ftp://ftp.ncbi.nih.gov/pub/geo/DATA/supplementary/series/GSE31519/GSE31519%5Fcomplete%5Fdataset%2Etxt%2Egz
#
# This last file from the GEO database has a rather complicated structure with a gap of 23 columns 
#   since it also contains samples from the ArrayExpress database which they have moved to the end.
#   The file is used to rebuild the complete dataset of 579 samples here:
#   

samples <- read.delim("GSE31519_complete_dataset.txt", colClasses= "character", skip=1, nrows=1, header=FALSE)   # read the sample names from row-2
data <- read.delim("GSE31519_complete_dataset.txt", skip=4,header=FALSE)  # read data starting row-5
colnames(data)=t(samples)       # add sample names from row-2 as column names
data=data[,!samples==""]        # remove blank gap-columns with no sample-title in row-2 from expr data
rm(samples)   # delete used variable
rownames(data)=data[,1]     # use first column with Affy IDs as rownames
cd.compl.579=data[,-1]    # remove first column with Affy IDs
rm(data)  # delete used variable
cd.compl.579=cd.compl.579[,sort(colnames(cd.compl.579))]   # sort columns by sample name

header.579 <- read.delim("GSE31519_TNBC_SampleInfo_BCR.txt",colClasses = "character", row.names="SampleNames",h=T)   # all sample information

#
# The following 11 data objects are included in 
#   the R workspace "R_analysis.RData" and referred to 
#   in the respective sections of the analyses:
#
# metagenes.probelists --> probesets of TNBC metagenes from 
#                          Rody et al. 2011, Breast Cancer Res, 
# SAM.delta.table  --> delta table from SAM analysis
# SAMgenes.0p3  -->  264 probesets from delta-0.3-SAM
# SAMgenes.0p5  -->   26 probesets from delta-0.5-SAM
# n235.SAMpoor.probes -->  probeset list of 235 up genes from SAM
#                        with delta 0.3  (poor prognosis markers) 
# n29.SAMgood.probes  -->  probeset list of 29 lo genes from SAM
#                        with delta 0.3  (good prognosis markers)
# sabatier.gl --> gene list of Sabatier et al.
#       source:  Sabatier et al. 2010 DOI 10.1007/s10549-010-0897-9
#                supplementary file 10549_2010_897_MOESM3_ESM.xls
# sdpp.gl   --> gene list of strom derived prognostic predictor
#       source: GeneSigDB Data Release 3 
#               http://compbio.dfci.harvard.edu/genesigdb/
#               signature "18438415-Table2.1" of Finak Nature Med 2008
#               24 EntrezGeneIDs corresp to 35 AffyProbes
# Tesch7.gl  --> gene list of Teschendorff-2008_7_immune-gene-signature
#       source:  Breast Cancer Research 2008, 10:R73
# wound.gl   --> gene list of wound response signature
#       source: GeneSigDB Data Release 3 
#               http://compbio.dfci.harvard.edu/genesigdb/
#               signature "14737219-CSRgenes" of Chang 2005 PNAS
#               592 EnsemblIDs corresp to 847 AffyProbes
# U133A.Affy.Entrez  --> link of Affymetrix probeset and Entrez IDs
#




# Supplementary Figure S1:  Selection of the TNBC finding cohort from multiple datasets based on dataset comparibility

#  Select the n=394 comparable TNBC samples from n=579 TNBC with normalized Affymetrix MAS5 data

#  Variables used:
#  cd.compl.579 ==> chip-data complete (data.frame, 579 samples in columns, 22283 probesets in rows)
#  header.579  ==> sample infos including dataset allocation (data.frame, 579 samples in columns)
#  n.probes ==> number of ProbeSets (rows in cd.compl.579)
#  n.col.cd ==> number of columns/samples in cd.compl.579
#  tcdm.579  ==> transposed chip data matrix
#  t.header.579 ==> transposed sample infos (data.frame)
#  tdsn.579  ==> transposed dataset allocation (numeric vector)
#

n.col.cd=ncol(cd.compl.579)
n.probes=nrow(cd.compl.579)
tdsn.579=as.numeric(t(header.579["datas_new09",]))   # transposed dataset vector
tcdm.579=t(cd.compl.579)   # transposed chip data matrix
t.header.579=as.data.frame(t(header.579))  # transposed sample info header as data.frame
ds.mean=by(tcdm.579, tdsn.579, mean)  # means within individual datasets, 
                                     #  List is sorted by numeric dataset tdsn.579
tcdm.mean=apply(tcdm.579,2,mean)
        #   generates named list of 22283 means, which can be indexed by tcdm.mean[probes]
tcdm.stdev=apply(tcdm.579,2, sd)   # the same for the StdDev


# Calculate comparability metrics for the datasets:
# (calculate sum of squared differences of dataset-mean from total-mean for all probesets)
# Define variables
n.datas=length(ds.mean)   # number of datasets
diff.to.mean= matrix(0,nrow=n.probes,ncol=n.datas) # matrix of differences from mean
nrm.diff.to.mean= matrix(0,nrow=n.probes,ncol=n.datas) # matrix of NORMALIZED diff from mean

for (probes in 1:n.probes)   # loop for all probesets
{

for (i in 1:n.datas)
# calculate for each dataset diff from global-mean of all datasets
# and save in matrix "diff.to.mean"
{
diff.to.mean[probes,i]=ds.mean[[i]][probes]- tcdm.mean[probes]
# again the same but normalize by dividing through StdDev
nrm.diff.to.mean[probes,i]=(ds.mean[[i]][probes]- tcdm.mean[probes]) / tcdm.stdev[probes]
}
}
# calculate squares of differences
squ.diff=diff.to.mean^2
squ.nrm.diff= nrm.diff.to.mean^2
# sum of squared differences by column
sum.squ.diff=apply(na.omit(squ.diff),2,sum)
sum.squ.nrm.diff=apply(na.omit(squ.nrm.diff),2,sum)
#  important: these vectors are still sorted by numeric dataset tdsn.579 !

# summarize results
comparab=data.frame(sort(unique(tdsn.579)),sum.squ.diff,sum.squ.nrm.diff)
names(comparab)=c("dataset","sum.squ.diff","sum.squ.nrm.diff")
sort.comparab=comparab[order(comparab$sum.squ.nrm.diff),]

# integrate normalized comparab data in sample info in t.header.579
for (i in 1:n.col.cd) 
    {t.header.579$comparab_nrm[i]= comparab$sum.squ.nrm.diff[comparab$dataset==tdsn.579[i]]}
# remove temporary variables:
rm(diff.to.mean, nrm.diff.to.mean, ds.mean,tcdm.mean, tcdm.stdev, squ.diff, squ.nrm.diff, sum.squ.diff, sum.squ.nrm.diff)

plot(sort(t.header.579$comparab_nrm),type="l")
abline(8000,0,col="red")


###################################################################################################





#  Select a subset of datasets with lowest comparability metric
# Select a subset of comparab by defining criteria:
compar.subset= subset(comparab, subset= sum.squ.nrm.diff < 8000)
# vector of corresponding datasets:
datas.subset=compar.subset$dataset
# generate logical vector FALSE/TRUE for the complete dataset of 579 TNBC:
subset.index.vector=(tdsn.579 %in% datas.subset)
# query selected samples from transposed chipdata matrix:
tcdm.394=tcdm.579[subset.index.vector , ]
# query  corresponding transposed dataset vector:
tdsn.394=tdsn.579[subset.index.vector]
# query selcted samples from NOT-transposed dataset (and corresponding header and t.header):
cd.compl.394= cd.compl.579[,subset.index.vector]
header.394= header.579[,subset.index.vector]
t.header.394= t.header.579[subset.index.vector,]
  


# Supplementary Figure S2: Analysis of a potential dataset bias among probesets

# Calculate Kruskal-Wallis statistics for all probesets according to their association with the dataset vector among the 394 TNBC samples:

# Variables used:    tcdm.394, tdsn.394, n.probes, cd.compl.394
# Newly generated variables:    kruskal.param

kruskal.result=kruskal.test(tcdm.394[,1],tdsn.394)  # Define variable as "list" by performing test once
kruskal.stat= matrix(0,nrow=n.probes)  # matrix for chi-statistics
kruskal.p=matrix(0,nrow=n.probes)  # matrix for p-values

# loop perfoming kruskal-test for each probeset (chip-data vs. dataset-vector)
# the results of each test are variables of type LIST, which are combined by indexing with [[i]]
for (i in 1:n.probes)
{
kruskal.result[[i]]=kruskal.test(tcdm.394[,i], tdsn.394)   # results as LIST
kruskal.stat[i]=kruskal.result[[i]]$statistic  # read statistics from LIST in matrix
kruskal.p[i]=kruskal.result[[i]]$p.value   # read p-values from LIST in matrix
}
# summarize statistics and p-values and probeset names in one dataframe:
kruskal.param= data.frame(row.names=rownames(cd.compl.394),kruskal.stat,kruskal.p)  # dataframe with results
# remove temporary variables:
rm(kruskal.result, kruskal.stat, kruskal.p)
  
hist(kruskal.param$kruskal.stat,breaks=30,col="dark blue") # histogram for all probesets



###################################################################################################


# Calculation of metagene-values for all 394 samples:
# Variable used from above:    cd.compl.394 , metagenes.probelists 
#
# New variables:
# metagenes.probelists ==>  lists of probesets representing the different metagenes
# n.metagenes ==> number of different metagenes
# metag.means ==> matrix for metagene means of all samples
# cd.probes.c ==> probeset-names as character from  rownames of cd.compl.394


n.metagenes= ncol(metagenes.probelists)
metag.means=matrix(0,ncol= ncol(cd.compl.394),nrow=n.metagenes) # define matrix for results
cd.probes.c =rownames(cd.compl.394)  # probeset-names as character

for (i in 1: n.metagenes)   # loop for metagen calculation
{
probes.of.metag=as.character(metagenes.probelists[,i])  # retrieve probes as character
# generate logical vector of selected probesets 
metag.subset.index.vector=(cd.probes.c %in% probes.of.metag) 
# calculate metagene-mean using logical vector (for all samples)
metag.mean.TF=by(as.matrix(cd.compl.394),metag.subset.index.vector,mean) 
    # provides mean values for TRUE (metagene probesets) and FALSE (not used)
metag.means[i,]= metag.mean.TF$'TRUE'
}
# Summarize results:
rownames(metag.means)=colnames(metagenes.probelists)
colnames(metag.means)=colnames(cd.compl.394)
metag.df=as.data.frame(t(metag.means))  # transposed as dataframe (to query metagenes by name)
# remove temporary variables:
rm(n.metagenes, metag.means, cd.probes.c, probes.of.metag, metag.subset.index.vector, metag.mean.TF)

# Kruskal-Wallis-stats of metagene-probesets:

metagene.kruskal.stats=vector(mode="list",length=ncol(metagenes.probelists))
names(metagene.kruskal.stats)=colnames(metagenes.probelists)
median.metagene.kruskal.stats=vector(mode = "numeric", length = ncol(metagenes.probelists))
names(median.metagene.kruskal.stats)=colnames(metagenes.probelists)

for (i in 1: ncol(metagenes.probelists))   # loop for kruskal-statistic of metagenes calculation
{
probes.of.metag=as.character(metagenes.probelists[,i])  # retrieve probes as character
metagene.kruskal.stats[[i]]=kruskal.param$kruskal.stat[rownames(kruskal.param[,]) %in% probes.of.metag]
median.metagene.kruskal.stats[i]=median(metagene.kruskal.stats[[i]])
}

metagene.kruskal =cbind(metagene.kruskal.stats,median.metagene.kruskal.stats)[order(median.metagene.kruskal.stats),]
# remove temporary variables:
rm(metagene.kruskal.stats, median.metagene.kruskal.stats, probes.of.metag, i)
boxplot(metagene.kruskal[,1])

 

#  SAM-Analysis
#
#  Variables used:    tcdm.394,  t.header.394
#
#  to free memory some variables might be temporarily removed for this section:
#     rm(cd.compl.394, cd.compl.579)
#  these variable can be regenerated for later analyses by:
#     cd.compl.394=as.data.frame(t(tcdm.394))
#     cd.compl.579=as.data.frame(t(tcdm.579))


#  extract sample set with FollUp (required for SAM):
follup.index.vector= !(t.header.394$ev120=="")  # indexvector TRUE for 297 with follUp of 394 
tcdm.394.follup=subset(tcdm.394, subset=follup.index.vector)  # extract 297 samples with follup
cdm.follup=t(tcdm.394.follup)          # chipdata of 297 follUp samples re-transposed

patdata.follup=subset(t.header.394, subset=follup.index.vector)  # sample info of 297 with follup
probenames=rownames(cdm.follup)  # All Affy_IDs 
# SAM analysis:
library(samr)
set.seed(84048)
samdata=list(x=cdm.follup, y=as.numeric(as.character(patdata.follup$fu_120)), censoring.status=as.numeric(as.character(patdata.follup$ev120)), geneid=probenames, genenames=probenames)

samr.obj=samr(samdata, resp.type="Survival", nperms=5)   # 10 min for 20 permutations

delta.table<- samr.compute.delta.table(samr.obj)

samr.plot(samr.obj, del=.3)
samr.plot(samr.obj, del=.5)

samr.compute.siggenes.table(samr.obj, 0.3, samdata, delta.table)
samr.compute.siggenes.table(samr.obj, 0.5, samdata, delta.table)

# The results of these analyses are included in the following variables:
#
#  SAM.delta.table --> delta table for SAM using the 297 samples with follUp
#  SAMgenes.0p5  -->  results for delta 0.5:  26 up genes#
#  SAMgenes.0p3  --> results for delta 0.3:  235 up and 29 down genes
# n235.SAMpoor.probes -->  probeset list of 235 up genes from SAM
#                        with delta 0.3  (poor prognosis markers) 
# n29.SAMgood.probes  -->  probeset list of 29 lo genes from SAM
#                        with delta 0.3  (good prognosis markers)


# remove variables:
rm(tcdm.394.follup, cdm.follup, patdata.follup, probenames, follup.index.vector, samdata, delta.table) 
# Suppl.Fig S5:   Analysis of dataset bias of metagenes and the prognostic signatures


#  New variable used: metagenes.probelists 
#  probe set lists of metagenes and prognostic signatures

# Variables used:    metagenes.probelists   kruskal.param
# New generated variables:    metagene.kruskal

metagene.kruskal.stats=vector(mode="list",length=ncol(metagenes.probelists))
names(metagene.kruskal.stats)=colnames(metagenes.probelists)
median.metagene.kruskal.stats=vector(mode = "numeric", length = ncol(metagenes.probelists))
names(median.metagene.kruskal.stats)=colnames(metagenes.probelists)

for (i in 1: ncol(metagenes.probelists))   # loop for kruskal-statistic of metagenes calculation
{
probes.of.metag=as.character(metagenes.probelists[,i])  # retrieve probes as character
metagene.kruskal.stats[[i]]=kruskal.param$kruskal.stat[rownames(kruskal.param[,]) %in% probes.of.metag]
median.metagene.kruskal.stats[i]=median(metagene.kruskal.stats[[i]])
}

metagene.kruskal =cbind(metagene.kruskal.stats,median.metagene.kruskal.stats)[order(median.metagene.kruskal.stats),]
# remove temporary variables:
rm(metagene.kruskal.stats, median.metagene.kruskal.stats, probes.of.metag, i)

# generate box plot of kruskal-stats of probesets of metagenes:

select.index=!names(metagene.kruskal[,1]) %in% c("SAM_264","SAM_26")
par(mar=c(7,4,4,2))
boxplot(metagene.kruskal[select.index,1], las=2)   # labels perpendicular to axis
 
# generate box plot of kruskal-stats of probesets of prognostic signatures
#    SAM_264 (delta 0.3) and SAM_26 (delta 0.5):

select.index=names(metagene.kruskal[,1]) %in% c("SAM_264","SAM_26")
par(mar=c(7,4,4,2))
boxplot(metagene.kruskal[select.index,1], las=2)   # labels perpendicular to axis
  
 
#  Supplementary Figure S6: Correlation of individual markers from the prognostic signatures with known metagenes in triple negative breast cancer 

# Variables used:
#  cd.compl.394 , n235.SAMpoor.probes , n29.SAMgood.probes

#  Supplementary Figure S6-C:   235 up genes, correlation-cutoff 0.3

n235.SAMpoor.probes_data=cd.compl.394[rownames(cd.compl.394) %in% n235.SAMpoor.probes[,],]
#calculate correlation of probesets to metagenes:
chipdata=n235.SAMpoor.probes_data 
length_chipdata=length(chipdata)
#  transpose chipdata
t.chipdata=as.data.frame(t(chipdata))
Cor.metagenes=matrix(data=NA, nrow=length(t.chipdata), ncol=0)  # define variable Cor.metagenes

# loop for selection of metagene
for (SelMetag  in c("IL8", "VEGF", "Prolif", "Basal", "CLDN3", "Apocrine", "Histone", "Adipocyte", "Stroma", "IFN", "MHC1", "T.Cell", "MHC2", "B.Cell", "Hemoglobin", "HOX"))
 {
  #  retrieve data of selected metagene 
  SelectedMetag.data= metag.df[SelMetag]
  #  Pearson-Korrelation of selected metagene and all probesets from list
  temp.cor=cor(SelectedMetag.data,t.chipdata,use="pairwise.complete.obs")
  Cor.metagenes=cbind(Cor.metagenes,t(temp.cor))
    # End of loop, use next metagene with already transposed chipdata
  }
 n235.SAMpoor.probes_Cor=Cor.metagenes

#  assign each probeset to metagene with highest correlation:
CorrMatrix= n235.SAMpoor.probes_Cor
# determin maximal absolute correlation for each probeset:
MaxAbsCorr= as.numeric(by(abs(CorrMatrix),c(1:nrow(CorrMatrix)),max))
# variablen definieren
indexvect=vector(mode='logical',length=ncol(CorrMatrix))
names=colnames(CorrMatrix)
namelist=vector(mode='character',length=nrow(CorrMatrix))
MaxCorr=vector(mode='numeric', length=nrow(CorrMatrix))

for (i in c(1:nrow(CorrMatrix))) 
   { indexvect= abs(CorrMatrix[i,])==MaxAbsCorr[i]
     namelist[i]=names[indexvect]
     MaxCorr[i]=CorrMatrix[i,indexvect]
   }

# additional number-coded-metagene-list for sorting
namelist.coded=namelist
metag.sort=c("IL8", "Histone", "CLDN3", "HOX", "Stroma", "Adipocyte", "Hemoglobin", "Prolif", "Basal", "Apocrine", "VEGF", "MHC1", "IFN", "MHC2", "T.Cell", "B.Cell")
for (i in c(1:16))
{ namelist.coded[grep(metag.sort[i], namelist)]=i
}
    # replace each metagene as ordered by number
n235.SAMpoor.probes_Cor = data.frame(n235.SAMpoor.probes_Cor, namelist, namelist.coded, MaxCorr, MaxAbsCorr, stringsAsFactors = FALSE)

# Generate probeset output table:

corr.cutoff=0.3   # cutoff for unclassified probesets with respect to metagene correlation

corr.heatmap=  n235.SAMpoor.probes_Cor
# replace metagene in namelist column with "unclassified" if MaxAbsCorr < corr.cutoff:
corr.heatmap[corr.heatmap[,"MaxAbsCorr"]<corr.cutoff,][,"namelist"]="unclassified"
# replace also metagene-code in namelist.coded column with "0" if MaxAbsCorr < corr.cutoff:
corr.heatmap[corr.heatmap[,"MaxAbsCorr"]<corr.cutoff,][,"namelist.coded"]=0
# sort corr.heatmap first by metagene and second by MaxAbsCorr
corr.heatmap = corr.heatmap[order(as.numeric(corr.heatmap[,"namelist.coded"]), corr.heatmap[,"MaxAbsCorr"]),,]   
# generate output table of selected columns:
out.table= corr.heatmap[, c("namelist",   "namelist.coded", "MaxCorr", "MaxAbsCorr")]
# export output table:
write.table(out.table, file="n235.SAMpoor.probes_Cor-table.txt", row.names=TRUE, col.names = NA, quote=FALSE, sep="\t")

# Generate correlation heatmap in R:
# sort probesets ASCENDING according metagenes namelist.coded for heatmap
sort.corr.heatmap = corr.heatmap[order(as.numeric(corr.heatmap[,"namelist.coded"]), corr.heatmap[,"MaxAbsCorr"],decreasing = TRUE),]   
# resort metagenes (columns) for heatmap and transpose for figure
sort.corr.heatmap=t(as.matrix(sort.corr.heatmap[,c("IL8", "Histone", "CLDN3", "HOX", "Stroma", "Adipocyte", "Hemoglobin", "Prolif", "Basal", "Apocrine", "VEGF", "MHC1", "IFN", "MHC2", "T.Cell", "B.Cell")]))
# generate heatmap-figure
heatmap(sort.corr.heatmap, Rowv = NA, Colv = NA, scale="none", main = "Correlation matrix",revC=TRUE,col= terrain.colors(100), margins = c(6, 1))

 
#  Supplementary Figure S6-A:   235 up genes, correlation-cutoff 0.3
# Generate probeset output table:

corr.cutoff=0.2   # cutoff for unclassified probesets with respect to metagene correlation

corr.heatmap=  n235.SAMpoor.probes_Cor
# replace metagene in namelist column with "unclassified" if MaxAbsCorr < corr.cutoff:
corr.heatmap[corr.heatmap[,"MaxAbsCorr"]<corr.cutoff,][,"namelist"]="unclassified"
# replace also metagene-code in namelist.coded column with "0" if MaxAbsCorr < corr.cutoff:
corr.heatmap[corr.heatmap[,"MaxAbsCorr"]<corr.cutoff,][,"namelist.coded"]=0
# sort corr.heatmap first by metagene and second by MaxAbsCorr
corr.heatmap = corr.heatmap[order(as.numeric(corr.heatmap[,"namelist.coded"]), corr.heatmap[,"MaxAbsCorr"]),,]   
# generate output table of selected columns:
out.table= corr.heatmap[, c("namelist",   "namelist.coded", "MaxCorr", "MaxAbsCorr")]
# export output table:
write.table(out.table, file="n235.SAMpoor.probes_Cor-table.txt", row.names=TRUE, col.names = NA, quote=FALSE, sep="\t")

# Generate correlation heatmap in R:
# sort probesets ASCENDING according metagenes namelist.coded for heatmap
sort.corr.heatmap = corr.heatmap[order(as.numeric(corr.heatmap[,"namelist.coded"]), corr.heatmap[,"MaxAbsCorr"],decreasing = TRUE),]   
# resort metagenes (columns) for heatmap and transpose for figure
sort.corr.heatmap=t(as.matrix(sort.corr.heatmap[,c("IL8", "Histone", "CLDN3", "HOX", "Stroma", "Adipocyte", "Hemoglobin", "Prolif", "Basal", "Apocrine", "VEGF", "MHC1", "IFN", "MHC2", "T.Cell", "B.Cell")]))
# generate heatmap-figure
heatmap(sort.corr.heatmap, Rowv = NA, Colv = NA, scale="none", main = "Correlation matrix",revC=TRUE,col= terrain.colors(100), margins = c(6, 4))
 

#  Supplementary Figure S6-D:   29 lo genes, correlation-cutoff 0.3

n29.SAMgood.probes_data=cd.compl.394[rownames(cd.compl.394) %in% n29.SAMgood.probes[,],]
#calculate correlation of probesets to metagenes (for 3 different lists of bimodal probesets):
chipdata=n29.SAMgood.probes_data 
length_chipdata=length(chipdata)
#  transpose chipdata
t.chipdata=as.data.frame(t(chipdata))
Cor.metagenes=matrix(data=NA, nrow=length(t.chipdata), ncol=0)  # define variable Cor.metagenes

# loop for selection of metagene
for (SelMetag  in c("IL8", "VEGF", "Prolif", "Basal", "CLDN3", "Apocrine", "Histone", "Adipocyte", "Stroma", "IFN", "MHC1", "T.Cell", "MHC2", "B.Cell", "Hemoglobin", "HOX"))
 {
  #  retrieve data of selected metagene 
  SelectedMetag.data= metag.df[SelMetag]
  #  Pearson-Korrelation of selected metagene and all probesets from list
  temp.cor=cor(SelectedMetag.data,t.chipdata,use="pairwise.complete.obs")
  Cor.metagenes=cbind(Cor.metagenes,t(temp.cor))
    # End of loop, use next metagene with already transposed chipdata
  }
 n29.SAMgood.probes_Cor=Cor.metagenes

#  assign each probeset to metagene with highest correlation:
CorrMatrix= n29.SAMgood.probes_Cor
# determin maximal absolute correlation for each probeset:
MaxAbsCorr= as.numeric(by(abs(CorrMatrix),c(1:nrow(CorrMatrix)),max))
# variablen definieren
indexvect=vector(mode='logical',length=ncol(CorrMatrix))
names=colnames(CorrMatrix)
namelist=vector(mode='character',length=nrow(CorrMatrix))
MaxCorr=vector(mode='numeric', length=nrow(CorrMatrix))

for (i in c(1:nrow(CorrMatrix))) 
   { indexvect= abs(CorrMatrix[i,])==MaxAbsCorr[i]
     namelist[i]=names[indexvect]
     MaxCorr[i]=CorrMatrix[i,indexvect]
   }

# additional number-coded-metagene-list for sorting
namelist.coded=namelist
metag.sort=c("IL8", "Histone", "CLDN3", "HOX", "Stroma", "Adipocyte", "Hemoglobin", "Prolif", "Basal", "Apocrine", "VEGF", "MHC1", "IFN", "MHC2", "T.Cell", "B.Cell")
for (i in c(1:16))
{ namelist.coded[grep(metag.sort[i], namelist)]=i
}
    # replace each metagene as ordered by number
n29.SAMgood.probes_Cor = data.frame(n29.SAMgood.probes_Cor, namelist, namelist.coded, MaxCorr, MaxAbsCorr, stringsAsFactors = FALSE)

# Generate probeset output table:

corr.cutoff=0.3   # cutoff for unclassified probesets with respect to metagene correlation

corr.heatmap=  n29.SAMgood.probes_Cor
# replace metagene in namelist column with "unclassified" if MaxAbsCorr < corr.cutoff:
corr.heatmap[corr.heatmap[,"MaxAbsCorr"]<corr.cutoff,][,"namelist"]="unclassified"
# replace also metagene-code in namelist.coded column with "0" if MaxAbsCorr < corr.cutoff:
corr.heatmap[corr.heatmap[,"MaxAbsCorr"]<corr.cutoff,][,"namelist.coded"]=0
# sort corr.heatmap first by metagene and second by MaxAbsCorr
corr.heatmap = corr.heatmap[order(as.numeric(corr.heatmap[,"namelist.coded"]), corr.heatmap[,"MaxAbsCorr"]),,]   
# generate output table of selected columns:
out.table= corr.heatmap[, c("namelist",   "namelist.coded", "MaxCorr", "MaxAbsCorr")]
# export output table:
write.table(out.table, file="n29.SAMgood.probes_Cor-table.txt", row.names=TRUE, col.names = NA, quote=FALSE, sep="\t")

# Generate correlation heatmap in R:
# sort probesets ASCENDING according metagenes namelist.coded for heatmap
sort.corr.heatmap = corr.heatmap[order(as.numeric(corr.heatmap[,"namelist.coded"]), corr.heatmap[,"MaxAbsCorr"],decreasing = TRUE),]   
# resort metagenes (columns) for heatmap and transpose for figure
sort.corr.heatmap=t(as.matrix(sort.corr.heatmap[,c("IL8", "Histone", "CLDN3", "HOX", "Stroma", "Adipocyte", "Hemoglobin", "Prolif", "Basal", "Apocrine", "VEGF", "MHC1", "IFN", "MHC2", "T.Cell", "B.Cell")]))
# generate heatmap-figure
heatmap(sort.corr.heatmap, Rowv = NA, Colv = NA, scale="none", main = "Correlation matrix",revC=TRUE,col= terrain.colors(100), margins = c(6, 1))

 
#  Supplementary Figure S6-B:   29 lo genes, correlation-cutoff 0.2
# Generate probeset output table:

corr.cutoff=0.2   # cutoff for unclassified probesets with respect to metagene correlation

corr.heatmap=  n29.SAMgood.probes_Cor
# replace metagene in namelist column with "unclassified" if MaxAbsCorr < corr.cutoff:
corr.heatmap[corr.heatmap[,"MaxAbsCorr"]<corr.cutoff,][,"namelist"]="unclassified"
# replace also metagene-code in namelist.coded column with "0" if MaxAbsCorr < corr.cutoff:
corr.heatmap[corr.heatmap[,"MaxAbsCorr"]<corr.cutoff,][,"namelist.coded"]=0
# sort corr.heatmap first by metagene and second by MaxAbsCorr
corr.heatmap = corr.heatmap[order(as.numeric(corr.heatmap[,"namelist.coded"]), corr.heatmap[,"MaxAbsCorr"]),,]   
# generate output table of selected columns:
out.table= corr.heatmap[, c("namelist",   "namelist.coded", "MaxCorr", "MaxAbsCorr")]
# export output table:
write.table(out.table, file="n29.SAMgood.probes_Cor-table.txt", row.names=TRUE, col.names = NA, quote=FALSE, sep="\t")

# Generate correlation heatmap in R:
# sort probesets ASCENDING according metagenes namelist.coded for heatmap
sort.corr.heatmap = corr.heatmap[order(as.numeric(corr.heatmap[,"namelist.coded"]), corr.heatmap[,"MaxAbsCorr"],decreasing = TRUE),]   
# resort metagenes (columns) for heatmap and transpose for figure
sort.corr.heatmap=t(as.matrix(sort.corr.heatmap[,c("IL8", "Histone", "CLDN3", "HOX", "Stroma", "Adipocyte", "Hemoglobin", "Prolif", "Basal", "Apocrine", "VEGF", "MHC1", "IFN", "MHC2", "T.Cell", "B.Cell")]))
# generate heatmap-figure
heatmap(sort.corr.heatmap, Rowv = NA, Colv = NA, scale="none", main = "Correlation matrix",revC=TRUE,col= terrain.colors(100), margins = c(6, 4))
 


 
# Figure 3: Relationship of the 264 probeset signature to the 16 metagenes and seven known prognostic signatures in TNBC
#
# Variables used:
# tcdm.394
# SAMgenes.0p3  ,  SAMgenes.0p5 , 
# n297TNBC_n26SAM_siggenes_0p5_up.txt , U133A.Affy.Entrez 
# sabatier.gl , sdpp.gl , Tesch7.gl , wound.gl
#

library(genefu)
library(simpleaffy)  # for pearson-corr based clustering

#  calculate several signature scores from tcdm.394

sabatier.score <- sig.score(x= sabatier.gl, data=tcdm.394, annot=U133A.Affy.Entrez, do.mapping=FALSE, signed=TRUE, verbose=TRUE)

Tesch7.score <- sig.score(x= Tesch7.gl, data=tcdm.394, annot=U133A.Affy.Entrez, do.mapping=FALSE, signed=TRUE, verbose=TRUE)

sdpp.score <- sig.score(x= sdpp.gl, data=tcdm.394, annot=U133A.Affy.Entrez, do.mapping=FALSE, signed=TRUE, verbose=TRUE)

wound.score <- sig.score(x= wound.gl, data=tcdm.394, annot=U133A.Affy.Entrez, do.mapping=FALSE, signed=TRUE, verbose=TRUE)

## make of ggi signature a gene list
ggi.gl <- cbind(sig.ggi[ ,c("probe", "EntrezGene.ID")], "coefficient"=ifelse(sig.ggi[ ,"grade"] == 1, -1, 1))
ggi.score <- sig.score(x= ggi.gl, data=tcdm.394, annot=U133A.Affy.Entrez, do.mapping=FALSE, signed=TRUE, verbose=TRUE)

gene70.score <- gene70(data=tcdm.394, annot=U133A.Affy.Entrez, do.mapping=TRUE, verbose = TRUE)

rs.394 <- oncotypedx(data= tcdm.394, annot= U133A.Affy.Entrez, do.mapping=FALSE, verbose=TRUE)

#  calculate SAM-prognostic-signature scores from tcdm.394 (Delta 0.3)

SAMchipdata= cd.compl.394 [rownames(cd.compl.394)%in%rownames(SAMgenes.0p3), ] [order(rownames(SAMgenes.0p3)),]   #  select the expression data of genes from the SAM-list

SAMscore= SAMchipdata [order(rownames(SAMchipdata)), ] *SAMgenes.0p3[order(rownames(SAMgenes.0p3)), ]$Score.d.   # product of expression values and SAM-Score as weighted score
SAMmean.264=mean(SAMscore)  # generates a named list of means of weighted SAM-scores for each sample
SAM_394.264=as.data.frame(SAMmean.264)

#  plot a cluster dendrogram of SAM-score and gene signatures in n394 sample cohort:

plot(standard.pearson(cbind(metag.df,sabatier.score$score, Tesch7.score$score, sdpp.score$score, wound.score$score, ggi.score$score, gene70.score$score, rs.394$score, SAM_394.264)))
 


#  Supplementary Figure S7: Relationship of the 26 probeset signature to the 16 metagenes and seven known prognostic signatures in TNBC

#   use alternative list of 26 stringent SAM-genes from delta 0.5
#    in analysis similar as above:
SAMscore= SAMchipdata[order(rownames(SAMchipdata)), ]* SAMgenes.0p5 [order (rownames (SAMgenes.0p5)), ]$Score.d.
SAMmean.26=mean(SAMscore)
SAM_394.26=as.data.frame(SAMmean.26)


#  plot a cluster dendrogram of SAM-score and gene signatures in n394 sample cohort:

plot(standard.pearson(cbind(metag.df,sabatier.score$score, Tesch7.score$score, sdpp.score$score, wound.score$score, ggi.score$score, gene70.score$score, rs.394$score, SAM_394.26)))
 
